import { Subject } from "./Subject";
import { SubjectGroup } from "./SubjectGroup";
import { ObjectLiteral } from "../../common/ObjectLiteral";
/**
 */
export declare class SubjectCollection extends Array<Subject> {
    /**
     * Pushes subject to the collection only in the case if subject with same entity target and entity id
     * does not exist in the collection.
     */
    pushIfNotExist(subject: Subject): void;
    /**
     * Finds subject with a given entity.
     */
    findByEntity(entity: ObjectLiteral): Subject | undefined;
    /**
     * Finds subject where entity like given entity.
     * Comparision made by entity id.
     */
    findByEntityLike(likeSubject: Subject): Subject | undefined;
    /**
     * Finds subject where entity like given subject's entity.
     * Comparision made by entity id.
     */
    findByEntityLike(entityTarget: Function | string, entity: ObjectLiteral): Subject | undefined;
    /**
     * Groups given Subject objects into groups separated by entity targets.
     */
    groupByEntityTargets(): SubjectGroup[];
    findByEntityId(entityTarget: Function | string, id: any): Subject | undefined;
    findByDatabaseEntityLike(entityTarget: Function | string, entity: ObjectLiteral): Subject | undefined;
    findByDatabaseId(entityTarget: Function | string, id: any): Subject | undefined;
    hasWithEntity(entity: ObjectLiteral): boolean;
}
