"use strict";
const Subject_1 = require("./Subject");
const SubjectCollection_1 = require("./SubjectCollection");
/**
 * Helps to create subject and subject collection objects.
 */
class SubjectFactory {
    /**
     * Creates and SubjectCollection object and fills it with all
     * unique entities from given entity and all its downside relations.
     */
    createCollectionFromEntityAndRelations(entity, metadata) {
        const subjects = new SubjectCollection_1.SubjectCollection();
        const recursive = (entity, metadata) => {
            subjects.push(new Subject_1.Subject(metadata, entity));
            metadata.extractRelationValuesFromEntity(entity, metadata.relations)
                .filter(([relation, value]) => relation.isCascadeInsert || relation.isCascadeUpdate || relation.isCascadeRemove)
                .filter(([relation, value]) => {
                return !subjects.find(subject => subject.entity === value);
            })
                .forEach(([relation, value]) => {
                recursive(value, relation.inverseEntityMetadata);
            });
        };
        recursive(entity, metadata);
        return subjects; // todo: compare resulted entities by id and throw exception if there are duplicates?
    }
}
exports.SubjectFactory = SubjectFactory;

//# sourceMappingURL=SubjectFactory.js.map
