"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const SubjectCollection_1 = require("../../persistment/subject/SubjectCollection");
const Subject_1 = require("../../persistment/subject/Subject");
/**
 * Transforms plain old javascript object
 * Entity is constructed based on its entity metadata.
 */
class DatabaseSubjectsLoader {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    constructor(connection) {
        this.connection = connection;
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     */
    load(subjectCollection) {
        return __awaiter(this, void 0, void 0, function* () {
            const subjects = new SubjectCollection_1.SubjectCollection();
            const promises = subjectCollection
                .groupByEntityTargets()
                .map(subjectGroup => {
                const allIds = subjectGroup.subjects.map(subject => subject.mixedId);
                const metadata = this.connection.getMetadata(subjectGroup.target);
                return this.connection
                    .getRepository(subjectGroup.target)
                    .findByIds(allIds, { alias: metadata.table.name, enabledOptions: ["RELATION_ID_VALUES"] })
                    .then(entities => {
                    entities.forEach(entity => {
                        subjects.push(new Subject_1.Subject(metadata, entity));
                    });
                });
            });
            yield Promise.all(promises);
            return subjects;
        });
    }
    loadRemoved(persistedSubjects) {
    }
}
exports.DatabaseSubjectsLoader = DatabaseSubjectsLoader;

//# sourceMappingURL=DatabaseSubjectsLoader.js.map
